/*
 * Compressed File Library 2.0
 * Complete rewrite over 1.x, based on good and bad points of old CFL.
 * Copyright (c) 1998 Jari Komppa
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "zlib.h"

#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))
#define CHECKSUM_ASCIIZ 0xffffffff

struct cflstruct {
  struct cflstruct * next;
  struct cflstruct * prev;
  char * dosfname;
  int fileofs;
};

struct resstruct {
  struct resstruct * next;
  struct resstruct * prev;
  struct cflstruct * cflfile;
  unsigned int hash;
  char * fname;
  int ofs;
  int len;
  int decompressed_len;
  int algo;
  int csum;
};

#pragma pack(1)
struct resinfostruct {
  int cflname;
  int ofs;
  int len;
  int decompressed_len;
  int algo;
  int csum;
};


#pragma pack()

static struct resstruct * resbase=NULL;
static struct cflstruct * cflbase=NULL;
int CFL_confirm_integrity=1;
char CFL_copyrite[]="** Compressed File Library 2.0 (c) 1998 Jari Komppa - Portions Copyright (c) 1998 Jari Komppa **";

static char * CFL_strdup(char * asciiz) {
  char * buf;
  int i;
  i=strlen(asciiz);
  buf=malloc(i+1);
  memset(buf,0,i+1);
  memcpy(buf,asciiz,i);
  return buf;
}

static void CFL_panic(char * reason) {
  reason=reason;
  exit(1);
}

static int CFL_preprocess(char * buf, int blen, int algo) {
  char * temp;
  if (algo==0)
    return 0;
  temp=malloc(blen);
  memcpy(temp,buf,blen);
  /* temp -> buf tnx */
  switch (algo) {
  default: break;
  }
  free(temp);
  return 0;
}

static int CFL_postprocess(char * buf, int blen, int algo) {
  char * temp;
  if (algo==0)
    return 0;
  temp=malloc(blen);
  memcpy(temp,buf,blen);
  /* temp -> buf tnx */
  switch (algo) {
  default: break;
  }
  free(temp);
  return 0;
}

static char * CFL_decompress(char * src, int srclen, int destlen, int algo) {
  char * outbuf;
  outbuf=malloc(destlen);
  switch (algo) {
  case 0: memcpy(outbuf,src,MIN(srclen,destlen)); break;
  case 1: /* 1 & 2 = zlib, only differs on compression */
  case 2: if (uncompress(outbuf,(unsigned long*)&destlen,src,srclen)!=Z_OK) {
            free(outbuf);
            outbuf=NULL;
          }
          break;
  }
  return outbuf;
}

#ifdef __WATCOMC__
unsigned int ROL(unsigned int i);
#pragma aux ROL="rol eax,3" parm [EAX] value [EAX] modify nomemory;
#endif

/*
 * Slightly different checksum, giving different values to
 * ABBA and BABA. Not quite CRC but certainly faster =)
 */
static unsigned int CFL_checksum(char * buf, unsigned int buflen) {
   unsigned int a,i=0;
#ifndef __WATCOMC__
  unsigned int q;
#endif
  if (buflen==CHECKSUM_ASCIIZ) { /* check ASCIIZ string instead */
    while (*buf) {
      i+=*buf;
#ifdef __WATCOMC__
      i=ROL(i);
#else
      q=i>>29; /* 3 high bits thankyou. */
      i<<=3;   /* practically this is ROL EAX,3 */
      i+=q;    /* instruction. C high level? yea rite! =) */
#endif
      buf++;
    }
  } else {
    for (a=0;a<buflen;a++) {
      i+=*(buf+a);
#ifdef __WATCOMC__
      i=ROL(i);
#else
      q=i>>29;
      i<<=3;
      i+=q;
#endif
    }
  }
  return i;
}

static char * CFL_depack(char * src, int srclen, int destlen, int algo, unsigned int csum) {
  char * outbuf;
  if (CFL_preprocess(src,srclen,(algo/100)%100)) {
    free(src);
    return NULL;
//    CFL_panic("Preprocessing failed");
  }
  if ((outbuf=CFL_decompress(src,srclen,destlen,algo%100))==NULL) {
    free(src);
    return NULL;
//    CFL_panic("Decompression failed");
  }
  if (CFL_postprocess(outbuf,destlen,algo/10000)) {
    free(src);
    free(outbuf);
    return NULL;
//    CFL_panic("Postprocessing failed");
  }
  if (CFL_confirm_integrity)
    if (CFL_checksum(outbuf,destlen)!=csum) {
      free(src);
      free(outbuf);
      return NULL;
//    CFL_panic("Checksum failed");
    }
  free(src);
  return outbuf;
}

static int CFL_dosexists(char * fname) {
  FILE * f;
  f=fopen(fname,"r");
  if (f==NULL) return 0;
  fclose(f);
  return 1;
}

 /*
  * This function opens target file, tries its best to find
  * a CFL 2.0 library somewhere, un-packs the library info
  * structure and returns it.
  * Used by CFL_opencfl and CFL_checkcfl.
  */
static char * CFL_openlibfile(char * fname, int ofs, struct cflstruct * cfld) {
  FILE * f;
  char * buf;
  int i[6];
  f=fopen(fname,"rb");
  fseek(f,ofs,SEEK_SET);
  /* try to identify header first at given offset. */
  fread((char*)i,4,1,f);
  if (*i!=0x324C4643) { /* "CFL2" reversed */
    /* So, header didn't match. Check the footter next. */
    fseek(f,-4,SEEK_END);
    fread((char*)i,4,1,f);
    if (*i!=0x4C464332) {/* "2CFL" reversed */
      fclose(f);
      return NULL; /* can't find CFL2 anywhere, bail out. */
    }
    fseek(f,-8,SEEK_END);
    fread((char*)i,4,1,f);
    fseek(f,-*i,SEEK_END);
    fread((char*)i,4,1,f);
    if (*i!=0x324C4643) { /* "CFL2" reversed */
      fclose(f);
      return NULL; /* footer matched but header didn't. TOU! */
    }
  }
  /*
   * If we get this far we should be 4 bytes from the CFL files offset.
   * So now we know some things; let's record these.
   */
  cfld->dosfname=CFL_strdup(fname);
  cfld->fileofs=ftell(f)-4;
  /* And now get all needed info from the header and decompress infostruc. */
  fread((char*)i,4,1,f);
  fseek(f,*i,SEEK_CUR); /* skip the infostring */
  fread((char*)i,4,5,f); /* read rest of header */
  buf=malloc(i[1]);
  fseek(f,cfld->fileofs+i[0],SEEK_SET);
  fread(buf,i[1],1,f);
  buf=CFL_depack(buf,i[1],i[2],i[3],i[4]);
  fclose(f);
  return buf; /* task completed, return buffer. */
}



static struct cflstruct * CFL_addcflstruct(struct cflstruct * cfld) {
  struct cflstruct * help, * data, * last;
  data=malloc(sizeof(struct cflstruct));
  memcpy(data,cfld,sizeof(struct cflstruct));
  if (cflbase==NULL) {
    cflbase=data;
    cflbase->next=NULL;
    cflbase->prev=NULL;
    return data;
  } else {
    help=cflbase;
    while ((help) && (strcmp(help->dosfname,data->dosfname))) {
      last=help;
      help=help->next;
    }
    if (help) { /* reopening already opened CFL */
      free(data->dosfname);
      free(data);
      return help;
    }
    last->next=data;
    data->prev=last;
    data->next=NULL;
    return data;
  }
}

static void CFL_addresource(struct resstruct * data) {
  struct resstruct * help, * last;
  if (resbase==NULL) {
    resbase=data;
    resbase->next=NULL;
    resbase->prev=NULL;
    return;
  } else {
    help=resbase;
    last=NULL;
    while ((help) && (!((help->hash==data->hash) && (strcmp(help->fname,data->fname)==0)))) {
      last=help;
      help=help->next;
    }
    if (help) { /* Replacement required */
      data->prev=help->prev;
      data->next=help->next;
      if (data->prev)
        data->prev->next=data;
      if (data->next)
        data->next->prev=data;
      free(help->fname);
      free(help);
      return;
    } /* new item */
    last->next=data;
    data->prev=last;
    data->next=NULL;
  }
}

char * CFL_CheckCFL(char * fname, int ofs) {
  char * buf, * restr;
  struct cflstruct cfld;
  int i;
  if (!CFL_dosexists(fname))
    return NULL;
  buf=CFL_openlibfile(fname,ofs,&cfld);
  if (buf==NULL)
    return NULL;
  i=*((int*)buf+2);
  restr=malloc(i+1);
  memset(restr,0,i+1);
  memcpy(restr,buf+12,i);
  free(buf);
  free(cfld.dosfname);
  return restr;
}

int CFL_OpenCFL(char * fname, int ofs) {
  char * buf;
  struct cflstruct cfli, * cfld;
  struct resinfostruct * ris;
  struct resstruct * res;
  int resources, fofs, strofs;
  if (!CFL_dosexists(fname))
    return 0;
  buf=CFL_openlibfile(fname,ofs,&cfli);
  if (buf!=NULL) {
    cfld=CFL_addcflstruct(&cfli); /* register cfl structure */
    resources=*(int*)(buf+4);
    fofs=*(int*)(buf);
    ris=(void*)(buf+fofs);
    strofs=fofs+resources*sizeof(struct resinfostruct);
    while (resources) {
      res=malloc(sizeof(struct resstruct));
      res->cflfile=cfld;
      res->fname=CFL_strdup(buf+ris->cflname+strofs);
      res->hash=CFL_checksum(res->fname,CHECKSUM_ASCIIZ);
      res->ofs=ris->ofs;
      res->len=ris->len;
      res->decompressed_len=ris->decompressed_len;
      res->csum=ris->csum;
      res->algo=ris->algo;
      CFL_addresource(res);
      ris++;
      resources--;
    }
    free(buf);
    return 1;
  }
  return 0;
}

static struct resstruct * CFL_findfile(char * fname) {
  struct resstruct * help;
  unsigned int i;
  char * dyn_fn=strdup(fname);
  _strupr(dyn_fn);
  help=resbase;
  if (!help)
    return NULL;
  i=CFL_checksum(dyn_fn,CHECKSUM_ASCIIZ);
  while ((help) && (!((help->hash==i)&&(strcmp(help->fname,dyn_fn)==0))))
    help=help->next;
  free(dyn_fn);
  return help;
}

int CFL_IsFile(char * fname) {
  if (CFL_findfile(fname)==NULL)
    return 0;
  return 1;
}

char * CFL_GetFile(char * fname) {
  struct resstruct * myfile;
  char * buf;
  FILE * f;
  myfile=CFL_findfile(fname);
  if (!myfile)
    return NULL;
  buf=malloc(myfile->len);
  f=fopen(myfile->cflfile->dosfname,"rb");
  fseek(f,myfile->cflfile->fileofs+myfile->ofs,SEEK_SET);
  fread(buf,myfile->len,1,f);
  fclose(f);
  return CFL_depack(buf,myfile->len,myfile->decompressed_len,myfile->algo,myfile->csum);
}

int CFL_GetFileB(char * fname, char * tbuf, int tbuflen) {
  struct resstruct * myfile;
  char * buf;
  FILE * f;
  myfile=CFL_findfile(fname);
  if (!myfile)
    return 0;
  buf=malloc(myfile->len);
  f=fopen(myfile->cflfile->dosfname,"rb");
  fseek(f,myfile->cflfile->fileofs+myfile->ofs,SEEK_SET);
  fread(buf,myfile->len,1,f);
  fclose(f);
  buf=CFL_depack(buf,myfile->len,myfile->decompressed_len,myfile->algo,myfile->csum);
  memcpy(tbuf,buf,MIN(tbuflen,myfile->decompressed_len));
  free(buf);
  return MIN(tbuflen,myfile->decompressed_len);
}

int CFL_GetFileSize(char * fname) {
  struct resstruct * myfile;
  myfile=CFL_findfile(fname);
  if (!myfile)
    return 0;
  return myfile->decompressed_len;
}

int CFL_GetCompressedFileSize(char * fname) {
  struct resstruct * myfile;
  myfile=CFL_findfile(fname);
  if (!myfile)
    return 0;
  return myfile->len;
}

FILE * CFL_GetFilePtr(char * fname) {
  struct resstruct * myfile;
  FILE * f;
  myfile=CFL_findfile(fname);
  if (!myfile)
    return NULL;
  f=fopen(myfile->cflfile->dosfname,"rb");
  fseek(f,myfile->cflfile->fileofs+myfile->ofs,SEEK_SET);
  return f;
}

void CFL_DropFilePtr(FILE * f) {
  fclose(f);
}

int CFL_GetFileOfs(char * fname) {
  struct resstruct * myfile;
  myfile=CFL_findfile(fname);
  if (!myfile)
    return -1;
  return myfile->cflfile->fileofs+myfile->ofs;
}

void CFL_CloseCFL(void) {
  struct cflstruct * cfld, * p;
  struct resstruct * res, * q;
  cfld=cflbase;
  res=resbase;
  while (cfld) {
    p=cfld;
    cfld=cfld->next;
    free(p->dosfname);
    free(p);
  }
  while (res) {
    q=res;
    res=res->next;
    free(q->fname);
    free(q);
  }
  cflbase=NULL;
  resbase=NULL;
}
