/* 
* Simple weekender shadow thingy
* d3dx7 single threaded app
* Sol/Trauma aka Jari Komppa, 2000
*
* All rights released.
* The author may not be held responsible of anything NEGATIVE that
* might be connected to this code.
*
*/

#define WIN32_LEAN_AND_MEAN		
#include <windows.h> // windows stuff
#include <stdio.h>   // standard IO
#include <stdlib.h>  // standard C lib
#include <ddraw.h>   // DirectDraw
#include <d3d.h>     // Direct3D
#include <d3dx.h>    // Direct3DX
#include "cfl2.h"		 // compressed file library
#include "fmod.h"

int fog=1;
int infos=1;		// flag to render info texts or not
int trees=90;		// no. of trees to render
int lights=1;		// no. of lights ot render
int currentblendmode=-1;	// check blend() function
#define MAXTREES 128	// max number of trees..
#define MAXLIGHTS 32	// ..and lights

// Float approximation of Pi
#define PI 3.1415926535897932384626433832795f
// Shorthand for setting blend modes
#define SET_BLEND(a,b) {d3dd->SetRenderState(D3DRENDERSTATE_DESTBLEND ,(b)); d3dd->SetRenderState(D3DRENDERSTATE_SRCBLEND ,(a));}
// defines for the three different blend modes used here
#define ALPHA_BLEND 1
#define MUL_BLEND 2
#define ADD_BLEND 3

// some typical variables..
char progname[]="Forest of shadows - Sol";
FSOUND_STREAM * music;
HWND mainhWnd;
HINSTANCE mainhInst;
LPD3DXCONTEXT dxctx;
LPDIRECT3DDEVICE7 d3dd;
LPDIRECT3D7 d3d;
LPDIRECTDRAW7 dd;
LPDIRECTDRAWSURFACE7 texFloor,      // floor texture, with alpha for gloss mapping
										 texTree,       // tree with alpha
										 texLight,      // light flare for the actual light flare and light on ground
										 texBackground, // background gradient
										 texInfo,       // infobox texture
										 texShadow;     // shadow for the trees

D3DXVECTOR3 eye; // position of the camera eye
D3DXVECTOR3 lite[MAXLIGHTS]; // position of lights

typedef struct tree_ {
	float x,y,z,scale;
} TREE;

TREE tree[MAXTREES];							// position and scale of trees
int sortid[MAXTREES+MAXLIGHTS];		// these two are for sorting the..
float sortz[MAXTREES+MAXLIGHTS];  // ..lights and trees

D3DXMATRIX billboardMatrix;	// matrix for rotation of billboards
D3DXMATRIX nailboardMatrix; // matrix for rotation of nailboards

// these two are in loader.cpp:
extern LPDIRECTDRAWSURFACE7 loadpcxtx(char * fname);
extern LPDIRECTDRAWSURFACE7 loadpcxtx(char * fname,char * alphafname);

// this function sets up the blending mode if different from currently
// active one: (Now, really correct way would be to check which ones work on 
// current device (using validatedevice) and fall back to more basic ones on
// those that don't - most of the stuff here could be done with just alpha_blend)
void blend(int blendmode) {
	if (currentblendmode==blendmode) return;
	currentblendmode=blendmode;
	
	if (blendmode==ALPHA_BLEND) {		
		d3dd->SetTextureStageState(0,D3DTSS_ALPHAOP,D3DTOP_MODULATE);
		d3dd->SetTextureStageState(0,D3DTSS_ALPHAARG1,D3DTA_TEXTURE);
		d3dd->SetTextureStageState(0,D3DTSS_ALPHAARG2,D3DTA_DIFFUSE);
		d3dd->SetTextureStageState(0,D3DTSS_COLOROP,D3DTOP_SELECTARG1);
		d3dd->SetTextureStageState(0,D3DTSS_COLORARG1,D3DTA_TEXTURE);
		d3dd->SetTextureStageState(0,D3DTSS_COLORARG2,D3DTA_CURRENT);
		SET_BLEND(D3DBLEND_SRCALPHA,D3DBLEND_INVSRCALPHA)
	}
	if (blendmode==MUL_BLEND) {
		d3dd->SetTextureStageState(0,D3DTSS_ALPHAOP,D3DTOP_DISABLE);
		d3dd->SetTextureStageState(0,D3DTSS_ALPHAARG1,D3DTA_TEXTURE);
		d3dd->SetTextureStageState(0,D3DTSS_ALPHAARG2,D3DTA_DIFFUSE);
		d3dd->SetTextureStageState(0,D3DTSS_COLOROP,D3DTOP_MODULATE);
		d3dd->SetTextureStageState(0,D3DTSS_COLORARG1,D3DTA_TEXTURE);
		d3dd->SetTextureStageState(0,D3DTSS_COLORARG2,D3DTA_CURRENT);
		SET_BLEND(D3DBLEND_ZERO,D3DBLEND_INVSRCCOLOR)
	}
	if (blendmode==ADD_BLEND) {
		d3dd->SetTextureStageState(0,D3DTSS_ALPHAOP,D3DTOP_SELECTARG2);
		d3dd->SetTextureStageState(0,D3DTSS_ALPHAARG1,D3DTA_TEXTURE);
		d3dd->SetTextureStageState(0,D3DTSS_ALPHAARG2,D3DTA_DIFFUSE);
		d3dd->SetTextureStageState(0,D3DTSS_COLOROP,D3DTOP_SELECTARG1);
		d3dd->SetTextureStageState(0,D3DTSS_COLORARG1,D3DTA_TEXTURE);
		d3dd->SetTextureStageState(0,D3DTSS_COLORARG2,D3DTA_CURRENT);
		SET_BLEND(D3DBLEND_SRCALPHA,D3DBLEND_ONE)
	}
}

// set up the vertex fog (for making ground and trees dark in distance)
void SetupVertexFog()
{
	float fStart = 1,    // linear fog distances
				fEnd   = 5; 

	// Disable fog blending as default (only activated in couple places)
	d3dd->SetRenderState(D3DRENDERSTATE_FOGENABLE, FALSE); 
	// Set the fog color.
	d3dd->SetRenderState(D3DRENDERSTATE_FOGCOLOR, 0);    
	// Set fog parameters.    
	d3dd->SetRenderState(D3DRENDERSTATE_FOGVERTEXMODE, D3DFOG_LINEAR);
	d3dd->SetRenderState(D3DRENDERSTATE_FOGSTART, *(DWORD *)(&fStart));
	d3dd->SetRenderState(D3DRENDERSTATE_FOGEND,   *(DWORD *)(&fEnd));

	d3dd->SetRenderState(D3DRENDERSTATE_RANGEFOGENABLE,TRUE);

}

// Render the background (skyline)
void renderBackground() {
	D3DTLVERTEX v[4];
	v[0].color=0xffffffff;
	v[0].specular=0xffffffff;
	v[0].rhw=1.0f;
	v[0].sx=0;
	v[0].sy=0;
	v[0].sz=0;
	v[0].tu=0;
	v[0].tv=0;
	v[3]=v[2]=v[1]=v[0];
	v[3].sx=v[1].sx=639;
	v[3].tu=v[1].tu=1;
	v[3].sy=v[2].sy=479;
	v[3].tv=v[2].tv=1;
	
	d3dd->SetTexture(0,texBackground);
	d3dd->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DFVF_TLVERTEX,v,4,0);
	d3dd->SetTexture(0,NULL);
}

// render the top corner info box 
void renderInfobox() {
	D3DTLVERTEX v[4];
	v[0].color=0xffffffff;
	v[0].specular=0xffffffff;
	v[0].rhw=1.0f;
	v[0].sx=0;
	v[0].sy=0;
	v[0].sz=0;
	v[0].tu=0;
	v[0].tv=0;
	v[3]=v[2]=v[1]=v[0];
	v[3].sx=v[1].sx=128;
	v[3].tu=v[1].tu=1;
	v[3].sy=v[2].sy=128;
	v[3].tv=v[2].tv=1;
	
	d3dd->SetTexture(0,texInfo);
  d3dd->SetTextureStageState(0,D3DTSS_MAGFILTER,D3DTFG_POINT); // kill trilinear 
  d3dd->SetTextureStageState(0,D3DTSS_MINFILTER,D3DTFG_POINT); // interpolation
	d3dd->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DFVF_TLVERTEX,v,4,0);
  d3dd->SetTextureStageState(0,D3DTSS_MAGFILTER,D3DTFG_LINEAR); // set up trilinear 
  d3dd->SetTextureStageState(0,D3DTSS_MINFILTER,D3DTFG_LINEAR); // interpolation
	d3dd->SetTexture(0,NULL);
}

// billboard sprite function, used here only to render the lights
void billboard(float x, float y, float z, float scale, int c) {
	D3DLVERTEX v[4];
	D3DXVECTOR4 vo[4],vt;
	
	// Set up the texture coords and colors:
	v[0].color=c;
	v[0].specular=0xffffffff;
	v[0].tu=1;
	v[0].tv=1;
	v[3]=v[2]=v[1]=v[0];
	v[3].tu=v[1].tu=0;
	v[3].tv=v[2].tv=0;
		
	// set up vertices:
	vo[0].x=-0.5f*scale;vo[0].y=-0.5f*scale;vo[0].z=0;vo[0].w=0;
	vo[1].x= 0.5f*scale;vo[1].y=-0.5f*scale;vo[1].z=0;vo[1].w=0;
	vo[2].x=-0.5f*scale;vo[2].y= 0.5f*scale;vo[2].z=0;vo[2].w=0;
	vo[3].x= 0.5f*scale;vo[3].y= 0.5f*scale;vo[3].z=0;vo[3].w=0;
	
	// transform and set vertices:
	for (int i=0;i<4;i++) {
		D3DXVec4Transform(&vt,&vo[i],&billboardMatrix);
		v[i].x=vt.x+x;
		v[i].y=vt.y+y;
		v[i].z=vt.z+z;
	}
	
	// render:
	d3dd->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DFVF_LVERTEX,v,4,0);
	
}

// billboard function that only rotates around Y for trees:
void nailboard(float x, float y, float z, float scale, int inv) {
	D3DLVERTEX v[4];
	D3DXVECTOR4 vo[4],vt;
	// Set up the texture coords and colors:
	if (inv<0) { // inv is -1 for mirror version of the tree
		v[0].color=0xbbffffff;
		v[0].specular=0xffffffff;
		v[0].tu=1;
		v[0].tv=0;
		v[3]=v[2]=v[1]=v[0];
		v[3].tu=v[1].tu=0;
		v[3].tv=v[2].tv=1;
	} else {
		v[0].color=0xffffffff;
		v[0].specular=0xffffffff;
		v[0].tu=1;
		v[0].tv=1;
		v[3]=v[2]=v[1]=v[0];
		v[3].tu=v[1].tu=0;
		v[3].tv=v[2].tv=0;
	}
	
	// set up vertices:
	vo[0].x=-0.5f*scale;vo[0].y=-0.5f*scale;vo[0].z=0;vo[0].w=0;
	vo[1].x= 0.5f*scale;vo[1].y=-0.5f*scale;vo[1].z=0;vo[1].w=0;
	vo[2].x=-0.5f*scale;vo[2].y= 0.5f*scale;vo[2].z=0;vo[2].w=0;
	vo[3].x= 0.5f*scale;vo[3].y= 0.5f*scale;vo[3].z=0;vo[3].w=0;
	
	// transform and set vertices:
	for (int i=0;i<4;i++) {
		D3DXVec4Transform(&vt,&vo[i],&nailboardMatrix);
		v[i].x=vt.x+x;
		v[i].y=vt.y+y*inv;
		v[i].z=vt.z+z;
	}
	
	// render:
  if (fog)
	  d3dd->SetRenderState(D3DRENDERSTATE_FOGENABLE, TRUE); 
	d3dd->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DFVF_LVERTEX,v,4,0);
  if (fog)
	  d3dd->SetRenderState(D3DRENDERSTATE_FOGENABLE, FALSE); 
}

// set up the camera and projection matrices.
void setupCamera(float f) {
	D3DXMATRIX camera;
	D3DXMATRIX fov;
	D3DXVECTOR3 at,up;
	eye.x=(float)sin(f)*2;
	eye.z=(float)cos(f)*4;
	eye.y=0.5f+(float)sin(f*1.1)*0.45f;
	eye.y=(float)sqrt(eye.y); // the camera prefers to stay high
	at.x=at.y=at.z=up.x=up.z=0;
	up.y=1;
	at.y=0.5f;
	D3DXMatrixLookAtLH(&camera,&eye,&at,&up);
	d3dd->SetTransform(D3DTRANSFORMSTATE_VIEW,(LPD3DMATRIX)&camera);
	D3DXMatrixPerspectiveFovLH(&fov,D3DXToRadian(45),1/1.2f,0.1f,6.0f); 
	d3dd->SetTransform(D3DTRANSFORMSTATE_PROJECTION,(LPD3DMATRIX)&fov);
}

// render the floor plane.
void renderPlane() {
	D3DLVERTEX v[4];
	v[0].color=0xffffffff;
	v[0].specular=0xffffffff;
	v[0].x=-5;
	v[0].y=0;
	v[0].z=-5;
	v[0].tu=-0.5f;
	v[0].tv=0.5f;
	v[3]=v[2]=v[1]=v[0];
	v[3].x=v[1].x=5;
	v[3].tu=v[1].tu=1.5f;
	v[3].z=v[2].z=5;
	v[3].tv=v[2].tv=2.5f;
  if (fog)
	  d3dd->SetRenderState(D3DRENDERSTATE_FOGENABLE, TRUE); 
	d3dd->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DFVF_LVERTEX,v,4,0);
  if (fog)
		d3dd->SetRenderState(D3DRENDERSTATE_FOGENABLE, FALSE); 
}

// render a shadow.
void shadow(float x, float z, float scale,float scaley,float dista,float rotation) {
	D3DLVERTEX v[4];
	float stretch=scale*(scaley*dista);
	
	int c=(int)((((0xff/lights)*3+1*0xff)/4)/pow(dista,1/2.0f));
	c*=0x01010101;
	int c2=(int)(0x33/pow(dista,2));
	c2*=0x01010101;
	v[0].color=c;
	v[0].specular=0xffffffff;
	v[0].tu=1;
	v[0].tv=1;
	v[3]=v[2]=v[1]=v[0];
	v[3].tu=v[1].tu=0;
	v[3].tv=v[2].tv=0;
	v[3].color=v[2].color=c2;
	
	D3DXMATRIX m;
	D3DXVECTOR4 vo[4],vt;	
	vo[0].x=-0.5f*scale;
	vo[0].z=0;
	vo[0].y=0;
	vo[0].w=0;
	vo[1].x=0.5f*scale;
	vo[1].z=0;
	vo[1].y=0;
	vo[1].w=0;
	vo[2].x=-0.5f*stretch;
	vo[2].z=stretch;
	vo[2].y=0;
	vo[2].w=0;
	vo[3].x=0.5f*stretch;
	vo[3].z=stretch;
	vo[3].y=0;
	vo[3].w=0;
	D3DXMatrixRotationY(&m,rotation);
	for (int i=0;i<4;i++) {
		D3DXVec4Transform(&vt,&vo[i],&m);
		//vt=vo[i];
		v[i].x=vt.x+x;
		v[i].z=vt.z+z;
		v[i].y=0;
		
	}
	d3dd->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DFVF_LVERTEX,v,4,0);
}

// render the light ring on the plane for each light
void spot(float x, float z, float scale,int base) {
	D3DLVERTEX v[4];
	int c=((base/lights)*3+1*base)/4;
	c*=0x01010101;
	v[0].color=c;
	v[0].specular=0xffffffff;
	v[0].x=-0.5f*scale+x;
	v[0].y=0;
	v[0].z=-0.5f*scale+z;
	v[0].tu=1;
	v[0].tv=1;
	v[3]=v[2]=v[1]=v[0];
	v[3].x=v[1].x=0.5f*scale+x;
	v[3].tu=v[1].tu=0;
	v[3].z=v[2].z=0.5f*scale+z;
	v[3].tv=v[2].tv=0;
	
	d3dd->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DFVF_LVERTEX,v,4,0);
}

// used to sort the trees and lights
int __cdecl qscomp(const void *elem1, const void *elem2 ) {
	float f=sortz[*(int*)elem2]-sortz[*(int*)elem1];
	if (f<0) return -1;
	if (f>0) return 1;
	return 0;
}

// sorts the trees and the lights
void sortTreesAndLights() {
	D3DXMATRIX m;
	D3DXVECTOR4 v;
	for (int i=0;i<trees+lights;i++)
		sortid[i]=i;
	d3dd->GetTransform(D3DTRANSFORMSTATE_VIEW,(LPD3DMATRIX)&m);
	
	for (i=0;i<trees;i++) {
		v.x=tree[i].x;
		v.y=tree[i].y;
		v.z=tree[i].z;
		v.w=0;
		D3DXVec4Transform(&v,&v,&m);
		sortz[i]=v.z;
	}
	for (i=0;i<lights;i++) {
		v.x=lite[i].x;
		v.y=lite[i].y;
		v.z=lite[i].z;
		v.w=0;
		D3DXVec4Transform(&v,&v,&m);
		sortz[i+trees]=v.z;
	}
	qsort(sortid,trees+lights,sizeof(int),&qscomp);
}

void setupSpriteMatrices() {
	// Set up inverse viewspace transformation matrix:	
	d3dd->GetTransform(D3DTRANSFORMSTATE_VIEW,(LPD3DMATRIX)&billboardMatrix);
	D3DXMatrixInverse(&billboardMatrix,NULL,&billboardMatrix);
	
	// Set up matrix that just rotates the trees towards us around Y:
	float f=(float)atan2(eye.x,eye.z);	
	D3DXMatrixRotationY(&nailboardMatrix,f); 
}

// The rendering main loop.
void reindeer() {
	float tick=GetTickCount()/1000.0f;
	int i,j,v;
	// we're not using zbuffer at all..
  dxctx->Clear(D3DCLEAR_TARGET); //|D3DCLEAR_ZBUFFER);
  
	d3dd->BeginScene();
	renderBackground();
	setupCamera(tick*0.2f);
	
	setupSpriteMatrices();

	// first, render the floor plane as solid texture:
	d3dd->SetTexture(0,texFloor);
	renderPlane(); 
	d3dd->SetTexture(0,NULL);
	
	// Set the light positions:
	for (i=0;i<lights;i++) {
		float f=(tick*0.3f+1000*(i+1));	  
		lite[i].x=(float)(sin(f*0.923)+cos(f*0.965)+sin(f*0.943))/3;
		lite[i].z=(float)(sin(f*0.913)+sin(f*0.985)+cos(f*0.943))/3;
		lite[i].y=(float)((cos(f*0.953)+sin(f*0.985)+cos(f*0.943))/6)+0.5f;
	}	
	
	// sort lights and trees:
	sortTreesAndLights();	
	
	// render reflections of lights and trees on top of the plane:
	d3dd->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE,TRUE);
	for (v=0;v<lights+trees;v++) {
		i=sortid[v];
		if (i<trees) {
			blend(ALPHA_BLEND);
			d3dd->SetTexture(0,texTree);
			nailboard(tree[i].x,tree[i].y,tree[i].z,tree[i].scale,-1);
			d3dd->SetTexture(0,NULL);
		} else {
			d3dd->SetTexture(0,texLight);
			blend(ADD_BLEND);
			billboard(lite[i-trees].x,-lite[i-trees].y,lite[i-trees].z,0.2f,0xbbffffff);
			d3dd->SetTexture(0,NULL);
		}
	}
	
	// render the plane again, this time with alpha (to mask out non-pond
	// reflections): (also known as gloss mapping)
	blend(ALPHA_BLEND);
	d3dd->SetTexture(0,texFloor);
	renderPlane();
	d3dd->SetTexture(0,NULL);

  // render shadows:
	blend(MUL_BLEND);
	d3dd->SetTexture(0,texShadow);
	for (i=0;i<trees;i++) {
		for (j=0;j<lights;j++) {
			float x=tree[i].x-lite[j].x;
			float z=tree[i].z-lite[j].z;
			shadow(tree[i].x,tree[i].z,tree[i].scale,
				1.0f/lite[j].y,
				(float)(fabs(x)+fabs(z)+1),
				(float)atan2(x,z)-PI*1.87f); 
			  // The tree bitmap is not straight, but since it rotates around Y axis
			  // it looks the same from every direction. That's why we rotate the shadow
			  // a bit to straighten it out.
		}
	}
	
	// render light hilights on ground:
	blend(ADD_BLEND);
	d3dd->SetTexture(0,texLight);
	for (i=0;i<lights;i++) {
		spot(lite[i].x,lite[i].z,5*(lite[i].y),(int)(0xff*pow(1-lite[i].y,3)));
	}
	d3dd->SetTexture(0,NULL);

  // render the trees and lights:	
	d3dd->SetTexture(0,NULL);
	for (v=0;v<lights+trees;v++) {
		i=sortid[v];
		if (i<trees) {
			blend(ALPHA_BLEND);
			d3dd->SetTexture(0,texTree);
			nailboard(tree[i].x,tree[i].y,tree[i].z,tree[i].scale,1);
			d3dd->SetTexture(0,NULL);
		} else {
			d3dd->SetTexture(0,texLight);
			blend(ADD_BLEND);
			billboard(lite[i-trees].x,lite[i-trees].y,lite[i-trees].z,0.2f,0xffffffff);
			d3dd->SetTexture(0,NULL);
		}
	}
	
	// and finally, render the info box:
	blend(ADD_BLEND);
	renderInfobox();
	d3dd->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE,FALSE);
  d3dd->EndScene();
	
}

LRESULT CALLBACK WindowProc(HWND hwnd,UINT uMsg,WPARAM wParam,LPARAM lParam) {
  if (uMsg==WM_DESTROY) {
		FSOUND_Stream_Stop(music);
    d3d->Release();
    d3dd->Release();
    D3DXUninitialize();
    exit(wParam);
  }
  return DefWindowProc(hwnd,uMsg,wParam,lParam);
}

// Initialization function..
void init(HWND hWnd, int fs) {
	int hicol=0;
	// primarily try for 32bpp triplebuffered, but fall back on
	// 16bpp doublebuffered if fails.
  if (FAILED(D3DXCreateContextEx(
		D3DX_HWLEVEL_RASTER,
		D3DX_CONTEXT_FULLSCREEN*fs,
		hWnd,
		NULL,
		32,
		D3DX_DEFAULT,
		D3DX_DEFAULT,
		D3DX_DEFAULT,
		2,
		640,
		480,
		D3DX_DEFAULT,
				&dxctx))) {
		hicol=1;
		if (FAILED(D3DXCreateContext(  
			D3DX_HWLEVEL_RASTER, 
			D3DX_CONTEXT_FULLSCREEN*fs,
			hWnd,
			640,  
			480,  
			&dxctx))) exit(2);
	}
  d3dd=dxctx->GetD3DDevice();
  d3d=dxctx->GetD3D();
  dd=dxctx->GetDD();
	
	// set up some defaults (note that dither is only enabled on 16bpp)

  d3dd->SetRenderState(D3DRENDERSTATE_CULLMODE,D3DCULL_NONE ); // no cull
  if (hicol)
		d3dd->SetRenderState(D3DRENDERSTATE_DITHERENABLE,TRUE); // dither on
	else
		d3dd->SetRenderState(D3DRENDERSTATE_DITHERENABLE,FALSE); // dither off
  d3dd->SetRenderState(D3DRENDERSTATE_ZENABLE,D3DZB_FALSE ); // zbuf off
	
  d3dd->SetRenderState(D3DRENDERSTATE_CLIPPING,TRUE);   // enable clipping
  d3dd->SetRenderState(D3DRENDERSTATE_LIGHTING,FALSE);  // disable lighting
	
  d3dd->SetTextureStageState(0,D3DTSS_MAGFILTER,D3DTFG_LINEAR); // set up trilinear 
  d3dd->SetTextureStageState(0,D3DTSS_MINFILTER,D3DTFG_LINEAR); // interpolation
  d3dd->SetTextureStageState(0,D3DTSS_MIPFILTER,D3DTFG_LINEAR);
	
	d3dd->SetTextureStageState(0,D3DTSS_ADDRESS,D3DTADDRESS_MIRROR); // mirroring of textures on wrap
	
	// load textures:
	texShadow=loadpcxtx("tree1.pcx");
	texTree=loadpcxtx("treetex1.pcx","tree1.pcx");
	texFloor=loadpcxtx("snow.pcx","snowmask.pcx"); // the ground was originally meant to be snow..
	texLight=loadpcxtx("flare.pcx");
	texBackground=loadpcxtx("skyline.pcx");
	texInfo=loadpcxtx("infobox.pcx");
	
	// set up tree positions:
	srand(0xc0cac01a);	// set random values to start from same seed every time
	for (int i=0;i<MAXTREES;i++) {
		tree[i].scale=(rand()/32768.0f)*0.75f+0.25f;
		tree[i].x=((rand()/16384.0f)-1)*1.75f;
		tree[i].y=tree[i].scale/2;
		tree[i].z=((rand()/16384.0f)-1)*1.75f;
	}
	
	// and finally set up the vertex fog:
	SetupVertexFog();
}

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow) 
{
  WNDCLASSEX winclass;
  HWND hWnd;
  MSG msg;
	
	// avoid warnings:
  lpCmdLine=lpCmdLine;
  hPrevInstance=hPrevInstance;
	
	// mount the CFL:
	if (!CFL_OpenCFL("shadows.cfl",0)) {
    MessageBox(NULL,"Filesystem mount failed.","Oh dear",MB_OK);
		exit(1); 
	}
	
	
  if (FAILED(D3DXInitialize())) return 0;
	
  mainhInst=hInstance;
	
  winclass.cbSize=sizeof(WNDCLASSEX);
  winclass.style=CS_DBLCLKS;
  winclass.lpfnWndProc=&WindowProc;
  winclass.cbClsExtra=0;
  winclass.cbWndExtra=0;
  winclass.hInstance=hInstance;
  winclass.hIcon=LoadIcon(mainhInst,MAKEINTRESOURCE(101));
  winclass.hCursor=LoadCursor(NULL,IDC_ARROW);
  winclass.hbrBackground=GetSysColorBrush(COLOR_APPWORKSPACE);
  winclass.lpszMenuName=NULL;
  winclass.lpszClassName=progname;
  winclass.hIconSm=NULL;
	
  if (!RegisterClassEx(&winclass))
    return 0;
	
  hWnd=CreateWindow(
    progname,
    progname,
    WS_SYSMENU|WS_CAPTION|WS_BORDER|WS_OVERLAPPED|WS_VISIBLE|WS_MINIMIZEBOX,
    CW_USEDEFAULT,
    0,
    640,
    480,
    NULL,
    NULL,
    hInstance,
    NULL);
	
  mainhWnd=hWnd;
  ShowWindow(hWnd,nCmdShow);
	int fs=0;
  if (MessageBox(NULL,"Shall we do it in fullscreen mode?","Silly question",MB_YESNO)==IDYES) fs=1;
	
	if (MessageBox(NULL,"Play music?","Another silly question",MB_YESNO)==IDYES) {
		if(!FSOUND_Init(44100,2,0)) return 0;
		music=FSOUND_Stream_OpenMpeg("taikamets.mp3", FSOUND_NORMAL | FSOUND_LOOP_NORMAL);
		if (music==NULL) return 0;
		FSOUND_Stream_Play(FSOUND_FREE,music);
	}
  init(hWnd,fs);	
	
  char str[256];
  int starttime,frame=0;
	float fps=0;
  starttime=GetTickCount();
  while (1) {
		if (fs)
			SetCursor(NULL);
		
		reindeer();
		if (infos) {
			frame++;
			int sec=GetTickCount()-starttime;
			if (sec>100) {
				fps=(frame*1000.0f)/sec;
				frame=0;
				starttime+=sec;
			}
			sprintf(str,"fps:%3.3f (i to hide info, f to toggle fog, esc quits)     Lights: %2d (z/x)   Trees: %3d (a/s)",fps,lights,trees);
			dxctx->DrawDebugText((float)(4/640.0),(float)(464/480.0),0,str); 
			dxctx->DrawDebugText((float)(2/640.0),(float)(462/480.0),0xffffff,str); 
		}
		Sleep(1); // give other applications a break
    dxctx->UpdateFrame(0);
		
    if(PeekMessage(&msg,hWnd,0,0,PM_REMOVE)) {
      switch(msg.message) {
      case WM_QUIT:
      case WM_DESTROY:
				FSOUND_Stream_Stop(music);
        d3d->Release();
        d3dd->Release();
        D3DXUninitialize();
        return (msg.wParam);
      case WM_KEYDOWN:
				switch( msg.wParam ) {
				case 'I': infos=1-infos; break;
				case 'A': if (trees>1) trees--; break;
				case 'S': if (trees<MAXTREES) trees++; break;
				case 'Z': if (lights>1) lights--; break;
				case 'X': if (lights<MAXLIGHTS) lights++; break;
				case 'F': fog=1-fog; break;
				case VK_ESCAPE:
				case VK_F12:
					PostMessage(hWnd, WM_CLOSE, 0, 0);
					break;
				}
				break;     
				default: 
					DefWindowProc(hWnd,msg.message,msg.wParam,msg.lParam);
      }          
    }  
  }    
}
